import 'package:easyflu/utils/object_util.dart';
import 'base_view_state_controller.dart';

class ProviderManager{

  static ProviderManager _instance = ProviderManager._internal();


  final _factories = Map<Type, dynamic>();

  final _factoriesByName = Map<String, dynamic>();

  ProviderManager._internal();


  factory ProviderManager() => _instance;


  void put(BaseViewStateProviderController controller, {String tag = ""}){
    if(!controller.disposed){
      if(ObjectUtil.isNotEmpty(tag)){
        _factoriesByName[tag] = controller;
      }else{
        _factories[controller.runtimeType] = controller;
      }
    }

  }


  T? find<T extends BaseViewStateProviderController>({String tag = ""}){
    if(ObjectUtil.isNotEmpty(tag)){
      if(_factoriesByName.containsKey(tag)){
        return _factoriesByName[tag];
      }
    }else{
      if(_factories.containsKey(T)){
        return _factories[T];
      }
    }
    return null;
  }


  void remove({String tag = "", BaseViewStateProviderController? controller}){
    if(ObjectUtil.isNotEmpty(tag) && _factoriesByName.containsKey(tag)){
      _factoriesByName.remove(tag);
    }else if(_factories.containsKey(controller.runtimeType)){
      _factories.remove(controller);
    }
  }


  void clear(){
    _factoriesByName.clear();
    _factories.clear();
  }

}